#!/usr/bin/python

# Class for running HMMtop and parsing the output to the 'standard' top-format we use (implemented).
# If a fastadir is given, the output parser inserts 'X' at the same positions as in the sequence since HMMTOP removes them.

import sys,re,os,shutil;

class hmmtop :
    def __init__(self) :
        self.ptnGetInfo = re.compile('>HP:\s+(\d+)\s+(>\w+)\s+(\w+)(\s+\d+)+');
        self.ptnGetHelices = re.compile('(\s+\d+)');

        self.hmmtopdir = os.path.dirname("/server/bin/hmmtop");
        self.hmmtopcmd = "./hmmtop -if=%s -sf=FAS -pi=spred -of=%s";

    def run(self, strFastaFile, strResFile) :
        os.system("cd %s; %s; cd - > /dev/null" % (self.hmmtopdir, self.hmmtopcmd % (strFastaFile,strResFile)));

    def run(self, strFastaFile, strResFile, print_probabilities = False) :
        prob_option = "";
        if print_probabilities :
            prob_option = "-pp";
        os.system("cd %s; %s %s; cd - > /dev/null" % (self.hmmtopdir, self.hmmtopcmd % (strFastaFile,strResFile), prob_option));

    def res2top_dir(self, strResdir, strTopdir, strFastadir=None) :
        lstResFiles = filter(lambda x:x.endswith('hmmtopres'),os.listdir(strResdir));
        for strElem in lstResFiles :
            strFastaFile = None;
            if strFastadir :
                strFastaFile = os.path.join(strFastadir, strElem[:-10] + '.fa');
            strResFile = os.path.join(strResdir, strElem);
            strOutFile = os.path.join(strTopdir, strElem[:-10] + '.hmmtop.top');
            self.res2top(strResFile,strOutFile, strFastaFile);

    def res2top(self, strResFile, strOutFile, strFastaFile=None) :
        flhRes = file(strResFile);
        for strLine in flhRes :
            if strLine.startswith('>HP:') :
                # Parse info line
                m = self.ptnGetInfo.match(strLine);
                intSeqLength = int(m.group(1));
                strName = m.group(2);
                strNterm = m.group(3);
                lstNums = [m.group(1).strip() for m in self.ptnGetHelices.finditer(strLine)];
                lstHelixPos = lstNums[2:];

                # Get predicted topology string
                # Init char to use in between TM-regions, TM-label, strTopo
                if ( strNterm == "IN" ) :
                    chrNonMembrane = 'i';
                else :
                    chrNonMembrane = 'o';
                chrMembrane = 'M';
                strTopo = '';

                # Snip off the two first elements in lstHelices and use the values to add one
                # non-TM region and one TM-region to strTopo. Pad strTopo with non-TM labels
                # when out of TM-regions until it is as long as the query sequence.
                while ( len(lstHelixPos) > 0 and len(lstHelixPos) % 2 == 0) :
                    intStart = int(lstHelixPos[0]);
                    intEnd = int(lstHelixPos[1]);
                    strTopo += chrNonMembrane * ( (intStart - 1) - len(strTopo));
                    strTopo += chrMembrane * (intEnd - intStart + 1);
                    # toggle
                    if chrNonMembrane == 'i' :
                        chrNonMembrane = 'o';
                    else :
                        chrNonMembrane = 'i';
                    # next round
                    lstHelixPos = lstHelixPos[2:];
                strTopo += chrNonMembrane * (intSeqLength - len(strTopo));

                # Compare to fasta file (hmmtop removes 'X's)
                if strFastaFile :
                    flhFasta = file(strFastaFile);
                    strSeq = "".join([s.strip() for s in flhFasta.readlines()[1:]]);
                    flhFasta.close();
                    for i in range(len(strSeq)) :
                        if strSeq[i] in 'Xx' :
                            strTopo = strTopo[:i] + strSeq[i] + strTopo[i:];

                # Print to file
                flhOut = file(strOutFile ,'w');
                print >>flhOut, strName;
                print >>flhOut, strTopo;
                flhOut.close();

                break ; # only one such line in the file

        flhRes.close();



if __name__ == '__main__' :
    
    # Check, parse $argv                                                                                  
    strUsage = "Usage: %s <fasta file (whole path)> <top file (outfile, whole path)>";
    if ( len(sys.argv) < len(strUsage.split('<')) ) :
        print strUsage % os.path.basename(sys.argv[0]);
        sys.exit(1);

    h = hmmtop();

    h.run(sys.argv[1],sys.argv[2] + '.hmmtopres');
    h.res2top(sys.argv[2] + '.hmmtopres', sys.argv[2]);
