#!/usr/bin/python

import sys,re,os;
import tempfile;
import subprocess;

class modhmm :
    def __init__(self) :
        pass ;
        
    class execModhmm :
        def __init__(self, lst, fastadir,outdir, blastdir, moddir, delculprits=False) :
            self.lstInfiles = lst;
            if os.path.isdir(fastadir) :
                self.strFastadir = fastadir;
            else :
                raise Exception("Directory '%s' doesn't exist" % fastadir);
            if os.path.isdir(outdir) :
                self.strOutdir = outdir;
            else :
                raise Exception("Directory '%s' doesn't exist" % outdir);
            if os.path.isdir(moddir) :
                self.strModdir = moddir;
            else :
                raise Exception("Directory '%s' doesn't exist" % moddir);
            self.lstModfiles = [os.path.join(self.strModdir,os.path.basename(s)[:-3] + '.mod') for s in self.lstInfiles];

            if os.path.isdir(blastdir) :
                self.strBlastdir = blastdir;
            else :
                raise Exception("Directory '%s' doesn't exist" % blastdir);

            self.blnDeleteFaultyResultFiles = delculprits;

            self.modhmm   = "/afs/pdc.kth.se/home/a/aronh/Public/tcomp/getopm/prodiv_tmhmm_v0.92b";
            self.modhmms = os.path.join(self.modhmm, "modhmm0.92b/modhmms");
            self.modhmms_command = "%s -m %s -s %s -f %s -o %s -r %s -L";
            self.pro_model   = os.path.join(self.modhmm, "HMM_FILES/PRO_TMHMM_0.92b.txt");
            self.prodiv_model   = os.path.join(self.modhmm, "HMM_FILES/PRODIV_TMHMM_0.92b.txt");
            self.s_model   = os.path.join(self.modhmm, "HMM_FILES/S_TMHMM_0.92b.txt");
            self.phobius_model   = os.path.join(self.modhmm, "HMM_FILES/phobius.txt");
            self.repl    = os.path.join(self.modhmm, "util/replacement_letter_multi.rpl");

            self.uniref = "/afs/pdc.kth.se/home/a/aronh/12vol/uniref90.fasta";
            self.blastall = "/afs/pdc.kth.se/home/a/aronh/1vol/blast-2.2.17/bin/blastall";
            self.blastcmd = "%s -p blastp -d %s -i %s -o %s -m 5";
            self.blast2mod = "/afs/pdc.kth.se/home/a/aronh/2vol/modhmm/msatomod-noexp.pl";


        def runBlast(self) :
            for strFastaFile in self.lstInfiles :
                strOutFile = os.path.join(self.strBlastdir, os.path.basename(strFastaFile)[:-3] + '.msa');
                os.system( self.blastcmd % (self.blastall,self.uniref,strFastaFile,strOutFile) );
                

        def msa2mod(self) :
            (intFoo, strNames) = tempfile.mkstemp(prefix='msa2mod',text='wt');
            flhName = file(strNames, 'w');
            print >>flhName, "\n".join([os.path.basename(s)[:-3] for s in self.lstInfiles]);
            flhName.close();
            os.system("%s %s %s %s %s" % ( self.blast2mod, strNames, self.strFastadir, self.strBlastdir, self.strModdir) );
            os.remove(strNames);
        
        def _makeNamefile(self, lst) :
            # Temporary file with filenames to modhmms
            (intFoo, strNames) = tempfile.mkstemp(prefix='modhmm',text='wt');
            flhNames = file(strNames, 'w');
            print >>flhNames, "\n".join(lst);
            flhNames.close();
            return strNames;

                        
        def runProdivTmhmm(self) :
            names = self._makeNamefile(self.lstModfiles);
            res = self._run(self.prodiv_model, 'msa', names);
            os.remove(names);
            return res;

        def runProTmhmm(self) :
            names = self._makeNamefile(self.lstModfiles);
            res = self._run(self.pro_model, 'msa', names);
            os.remove(names);
            return res;

        def runSTmhmm(self) :
            names = self._makeNamefile(self.lstInfiles);
            res = self._run(self.s_model, 'fa', names);
            os.remove(names);
            return res;

        def runPhobius(self) :
            names = self._makeNamefile(self.lstModfiles);
            res = self._run(self.phobius_model, 'msa', names);
            os.remove(names);
            return res;
            
        def _run(self, mdl, input, strNames) :
            # Run modhmms
            # parameters?
            strCommand = self.modhmms_command % \
                (self.modhmms,\
                     mdl,\
                     strNames,\
                     input,\
                     self.strOutdir,\
                     self.repl);
            flhOutput = subprocess.Popen(strCommand, shell=True, stdout=subprocess.PIPE).stdout;
            result = flhOutput.read();
            flhOutput.close();
            
            return result;

        def _isValidResult(self, filename) :
            blnOk = False;
            flhRes = file(filename);
            if ( 'Labeling' in flhRes.read() ) :
                blnOk = True;
            flhRes.close();
            return blnOk;

        def _getOutfile(self, infile) :
            return os.path.join(self.strOutdir, os.path.basename(infile) + '.res');

        def updateInfiles(self) :
            lstTodo = [];
            strCulprit = "";
            # Check each (potential) outfile
            for f in self.lstInfiles :
                strOutfile = self._getOutfile(f);
                # outfile exists, but isn't ok
                if ( os.path.isfile(strOutfile) and not self._isValidResult(strOutfile) ) :
                    flhRes = file(strOutfile);
                    text = flhRes.read();
                    flhRes.close();
                    # The one we choked on; don't run this one again
                    if ( 'Seq length:' in text ) :
                        strCulprit = f;
                    # The ones we just didn't get to; run them again
                    else :
                        lstTodo.append(f);
            self.lstInfiles = lstTodo;

            print "Couldn't predict '%s'" % strCulprit;
            if ( self.blnDeleteFaultyResultFiles ) :
                os.remove(self._getOutfile(strCulprit));

        def isDone(self) :
            for f in self.lstInfiles :
                strOutfile = self._getOutfile(f);
                if ( not ( os.path.isfile(strOutfile) and self._isValidResult(strOutfile) ) ) :
                    return False;
            return True;


    def res2top(self, strResFile, strTopFile) :
        ptnSeqName = re.compile("# Scores for sequence: '([\w-]+)");
        ptnStartLabels = re.compile("Labeling:");
        ptnEndLabels = re.compile("Posterior probabilities");

        flhResult = file(strResFile);

        strLine = flhResult.readline();
        m = ptnSeqName.match(strLine);
        if not m :
            print strResFile, ": SeqName match failed";
            flhResult.close();
            return ;
        else :
            strSeqName = m.group(1);

        # FF to topology labelling
        strLine = flhResult.readline();
        while ( not ptnStartLabels.match(strLine) ) :
            strLine = flhResult.readline();
        # Get topology
        strLine = flhResult.readline();
        strTopo = '';
        while ( not ptnEndLabels.match(strLine) ):
            strTopo += strLine.strip();
            strLine = flhResult.readline();
        
        flhResult.close();

        # Write to output file
        flhOutput = file(strTopFile, 'w');
        print >>flhOutput, ">%s" % strSeqName;
        print >>flhOutput, "%s" % strTopo;
        flhOutput.close();


if ( __name__ == '__main__' ) :
    # Check, parse $argv
    strUsage = "Usage: %s <fasta-file dir (in)> <result dir (out)> <top dir (out)> [delete faulty outfiles = 0]";
    if ( len(sys.argv) < len(strUsage.split('<')) ) :
        print strUsage % os.path.basename(sys.argv[0]);
        sys.exit(1);

    strIndir  = sys.argv[1].rstrip('/');
    strOutdir = sys.argv[2].rstrip('/');
    strTopdir = sys.argv[3].rstrip('/');

    blnDel = ( len(sys.argv) >= 4 ) and ( sys.argv[3] == "True" or \
                                          sys.argv[3] == "T" or \
                                          sys.argv[3] == "t" or \
                                          sys.argv[3] == "1" or \
                                          sys.argv[3] == "y" );

    print "Running prodiv_tmhmm on sequences in %s" % strIndir;

    # 0 List infiles
    lstFastafiles = [os.path.join(strIndir,s) for s in filter(lambda x:re.match('.*\.fa$',x), os.listdir(strIndir))];
    lstFastafiles.sort();

    # 1 Run prodiv-tmhmm
    print "Init of modhmm-object";
    m = modhmm();
    print "Init of execModhmm-object";
    tmhmm = m.execModhmm(lstFastafiles,strIndir,strOutdir, strOutdir, strOutdir, blnDel);
    print "Run blast";
    tmhmm.runBlast();
    print "Run msa2mod";
    tmhmm.msa2mod();
    print "Run modhmms";
    tmhmm.runProdivTmhmm();
##         tmhmm.runPhobius();
##         tmhmm.runProTmhmm();
##         tmhmm.runSTmhmm();

    # 2 Check outfiles, find out if everything was run
    while ( not tmhmm.isDone() ) :
        print "Not done";
        # Identify culprit
        # Shorten infiles list
        tmhmm.updateInfiles();
        tmhmm.runProdivTmhmm();
##         tmhmm.runPhobius();
##         tmhmm.runProTmhmm();
##         tmhmm.runSTmhmm();


    # 3 Parse outfiles
    for res in [os.path.join(strOutdir,s) for s in filter(lambda x:re.match('.*\.mod.res$',x), os.listdir(strOutdir))] :
        top = os.path.join(strTopdir, os.path.basename(res)[:-8] + '.top');
        m.res2top(res,top);

